==> verttice_ui-0.0.89.tar.gz: lib/src/widgets/data_factory/table_factory/table_content.dart <==
import "package:collection/collection.dart";
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/const/box_sizes.dart';
import "package:verttice_ui/src/const/icons.dart";
import 'package:verttice_ui/src/state/verttice_ui_state.dart';
import "package:verttice_ui/src/widgets/data_factory/form_factory/form_fields.dart";
import "package:verttice_ui/src/widgets/themed_toast.dart";
import "package:data_table_2/data_table_2.dart";
import "package:flutter/material.dart";
import "package:flutter/scheduler.dart";
import "package:flutter/services.dart";
import "package:provider/provider.dart";

import '../common/common_index.dart';
import 'filter_view.dart';

class TableContent extends StatefulWidget {
  final bool showCopyRowButton;
  final List<FormItem> columns;
  final List<Map<String, dynamic>> data;
  final int currentPage;
  final int pageLength;
  final void Function(
    Map<String, dynamic>? rowData,
    int rowIndex,
  ) handleSelectedRow;
  final void Function(List<Map<String, dynamic>>)? selectedListCallback;
  final Function(Map<String, List<FilterField>>) onFilterChanged;
  final Map<String, List<FilterField>> filters;
  final String uniqueIdField;
  final void Function({
    required String field,
    required bool asc,
  })? onSortChanged;

  const TableContent({
    Key? key,
    required this.showCopyRowButton,
    required this.columns,
    required this.data,
    required this.currentPage,
    required this.pageLength,
    required this.handleSelectedRow,
    required this.filters,
    required this.onFilterChanged,
    required this.uniqueIdField,
    this.onSortChanged,
    this.selectedListCallback,
  }) : super(key: key);

  @override
  _TableContentState createState() => _TableContentState();
}

class _TableContentState extends State<TableContent> {
  late List<Map<String, dynamic>> _data;
  final List<Map<String, dynamic>> _selctedRows = [];
  late final List<FormItem> _columns = [];
  late double _tableWidth;
  late int _currentPage;
  late int _pageLength;
  String? _selectedRow;
  int _currentSortColumn = 0;
  bool _isAscending = true;
  late Map<String, List<FilterField>> _filters;

  @override
  void initState() {
    super.initState();
    _filters = widget.filters;
    _data = widget.data;
    _currentPage = widget.currentPage;
    _pageLength = widget.pageLength;
    if (widget.showCopyRowButton) {
      _columns.add(FormItem(field: NoneFF()));
    }
    if (widget.selectedListCallback != null) {
      _columns.add(FormItem(field: NoneFF()));
    }
    _columns.addAll(
      widget.columns.where((item) => item.field.showOnTable).toList(),
    );
    _tableWidth =
        _columns.map((e) => e.field.tableColumnWidth + 17).toList().sum;
  }

  @override
  void didUpdateWidget(covariant TableContent oldWidget) {
    super.didUpdateWidget(oldWidget);

    if (!const DeepCollectionEquality().equals(_data, widget.data)) {
      _data = widget.data;
    }

    if (!const MapEquality().equals(_filters, widget.filters)) {
      _filters = widget.filters;
    }

    if (_currentPage != widget.currentPage ||
        _pageLength != widget.pageLength) {
      _currentPage = widget.currentPage;
      _pageLength = widget.pageLength;
    }
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      SchedulerBinding.instance?.addPostFrameCallback((_) {
        super.setState(fn);
      });
    }
  }

  List<DataColumn2> _buildColumns({
    required Map<String, List<FilterField>> filters,
  }) {
    final List<DataColumn2> tableColumns = [];
    for (var column in _columns) {
      if (_columns.indexOf(column) == 0 && widget.showCopyRowButton) {
        tableColumns.add(
          const DataColumn2(
            size: ColumnSize.S,
            label: SelectableText(
              "Copy",
            ),
          ),
        );
      } else if (widget.selectedListCallback != null &&
          ((!widget.showCopyRowButton && _columns.indexOf(column) == 0) ||
              _columns.indexOf(column) == 1)) {
        tableColumns.add(
          const DataColumn2(
            size: ColumnSize.S,
            label: SelectableText(
              "Select",
            ),
          ),
        );
      } else if (column.field.showOnTable &&
          column.field.displayName != "" &&
          column.field.name != "") {
        tableColumns.add(
          _builDataColumn(
            column: column,
            filters: filters,
          ),
        );
      }
    }
    return tableColumns;
  }

  DataColumn2 _builDataColumn({
    required FormItem column,
    required Map<String, List<FilterField>> filters,
  }) {
    return DataColumn2(
      size: column.field.tableColumnWidth <= 100
          ? ColumnSize.S
          : column.field.tableColumnWidth <= 150
              ? ColumnSize.M
              : ColumnSize.L,
      onSort: (int columnIndex, _) {
        setState(() {
          _currentSortColumn = columnIndex;
          _isAscending = !_isAscending;

          if (widget.onSortChanged != null) {
            widget.onSortChanged!(
              field: _columns[columnIndex].field.name,
              asc: _isAscending,
            );
          } else {
            _handleSortData(
              _columns[columnIndex],
              _columns[columnIndex].field.name,
              _isAscending,
            );
          }
        });
      },
      label: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          sizedBoxH4,
          SizedBox(
            height: 30,
            child: SelectableText(
              column.field.displayName,
              style: const TextStyle(
                overflow: TextOverflow.fade,
              ),
            ),
          ),
          SizedBox(
            height: 30,
            child: TextButton.icon(
              icon: Tooltip(
                message: "Apply filter",
                child: ThemedIcons.filter(
                  iconStyle: IconStyle(
                    size: 17,
                  ),
                ),
              ),
              label: Builder(
                builder: (context) {
                  final labletext = filters[column.field.name]
                      ?.map(
                        (e) => visualDataProcessor(
                          column,
                          {
                            column.field.name: e.value,
                          },
                          lookForPathToFieldOrItemBuilder: false,
                          igonorItemType: true,
                          context: context,
                        ).value,
                      )
                      .toList()
                      .join(", ");
                  return Tooltip(
                    message: labletext ?? '',
                    child: Text(
                      labletext ?? '',
                    ),
                  );
                },
              ),
              onPressed: () => _filterView(
                column: column,
                filters: filters,
              ),
            ),
          ),
          sizedBoxH2,
        ],
      ),
    );
  }

  Future<void> _filterView({
    required FormItem column,
    required Map<String, List<FilterField>> filters,
  }) async {
    await showDialog(
      context: context,
      useSafeArea: true,
      builder: (BuildContext context) {
        return SimpleDialog(
          insetPadding: const EdgeInsets.all(6),
          contentPadding: const EdgeInsets.all(6),
          children: [
            SizedBox(
              height: 450,
              width: 400,
              child: Scaffold(
                body: Center(
                  child: FilterView(
                    column: column,
                    filter: filters[column.field.name],
                    clearFilterCallback: () {
                      final updatedFilters = filters;
                      updatedFilters.remove(column.field.name);
                      widget.onFilterChanged(updatedFilters);
                    },
                    filterCallback: (List<FilterField> filter) {
                      final updatedFilters = filters;
                      updatedFilters[column.field.name] = filter;
                      widget.onFilterChanged(updatedFilters);
                    },
                  ),
                ),
              ),
            )
          ],
        );
      },
    );
  }

  void _handleSortData(
    FormItem column,
    String name,
    bool desc,
  ) {
    _data.sort((a, b) {
      dynamic aS = a[name] ?? "";
      dynamic bS = b[name] ?? "";
      if (column.field.pathToField != null &&
          column.field.pathToField!.isNotEmpty) {
        aS = fieldFromPathToField(
          column.field.pathToField!,
          a.cast<String, dynamic>(),
        );
        bS = fieldFromPathToField(
          column.field.pathToField!,
          b.cast<String, dynamic>(),
        );
      }

      if (aS == null || aS == "") {
        if (bS == null || bS == "") {
          return 0;
        } else {
          return 1;
        }
      } else {
        if (bS == null || bS == "") {
          return -1;
        }
      }

      var r = desc
          ? bS.toString().compareTo(aS.toString())
          : aS.toString().compareTo(bS.toString());
      if (r != 0) return r;
      return desc
          ? bS.toString().compareTo(aS.toString())
          : aS.toString().compareTo(bS.toString());
    });
  }

  DataRow _dataRowBuilder(
    String? selectedRow,
    int rowIndex,
    Map<String, dynamic> rowData,
    bool isDarkTheme,
  ) {
    final List<DataCell> currenRow = [];

    for (var column in _columns) {
      if (_columns.indexOf(column) == 0 && widget.showCopyRowButton) {
        currenRow.add(
          DataCell(
            _copyRow(rowData),
          ),
        );
      } else if (widget.selectedListCallback != null &&
          ((!widget.showCopyRowButton && _columns.indexOf(column) == 0) ||
              _columns.indexOf(column) == 1)) {
        final val = _selctedRows.firstWhereOrNull(
                (element) => const MapEquality().equals(element, rowData)) !=
            null;
        currenRow.add(
          DataCell(
            Tooltip(
              message: val ? "Uncheck" : "Check",
              child: Checkbox(
                onChanged: (selcted) {
                  setState(() {
                    if (selcted ?? false) {
                      _selctedRows.add(rowData);
                    } else {
                      _selctedRows.remove(rowData);
                    }

                    widget.selectedListCallback!(_selctedRows);
                  });
                },
                value: val,
              ),
            ),
          ),
        );
      } else if (column.field.showOnTable &&
          column.field.displayName != "" &&
          column.field.name != "") {
        final visualData = visualDataProcessor(
          column,
          rowData,
          context: context,
        );

        final widget = visualData.widget;
        currenRow.add(
          DataCell(widget),
        );
      }
    }

    final rowID = rowData[widget.uniqueIdField];

    return DataRow(
      onSelectChanged: (bool? selected) {
        widget.handleSelectedRow(rowData, rowIndex);

        setState(() {
          _selectedRow = rowID;
        });
      },
      color: selectedRow == rowID && rowID != null
          ? MaterialStateProperty.all(
              isDarkTheme ? Colors.black : Colors.grey[400],
            )
          : ((rowIndex % 2) == 0)
              ? MaterialStateProperty.all(
                  isDarkTheme ? Colors.grey[600] : Colors.grey[50],
                )
              : MaterialStateProperty.all(
                  isDarkTheme ? Colors.grey[800] : Colors.grey[200],
                ),
      cells: currenRow,
    );
  }

  Widget _copyRow(Map<String, dynamic> rowData) {
    return Tooltip(
      message: "Copy row",
      child: IconButton(
        onPressed: () async {
          await Clipboard.setData(
            ClipboardData(
              text: widget.columns
                  .map(
                    (e) => "${e.field.displayName}: ${visualDataProcessor(
                      e,
                      rowData,
                      context: context,
                    ).value}",
                  )
                  .toList()
                  .join(',\n'),
            ),
          );
          ThemedToast.info(
            context: context,
            message: "Row copied to clipboard",
          );
        },
        icon: ThemedIcons.copy(),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final dataLength = _data.length;

    final sublist = _data.sublist(
        (_currentPage * _pageLength) > dataLength
            ? dataLength
            : (_currentPage * _pageLength),
        ((_currentPage + 1) * _pageLength) > dataLength
            ? dataLength
            : ((_currentPage + 1) * _pageLength));

    return Consumer<VertticeUIState>(builder: (context, value, _child) {
      return DataTable2(
        showBottomBorder: true,
        scrollController: ScrollController(),
        minWidth: _tableWidth,
        sortColumnIndex: _currentSortColumn,
        sortAscending: _isAscending,
        headingRowHeight: 68,
        dataRowHeight: 55,
        columnSpacing: 1,
        columns: _buildColumns(
          filters: _filters,
        ),
        rows: sublist.mapIndexed((rowIndex, rowData) {
          return _dataRowBuilder(
            _selectedRow,
            rowIndex,
            rowData,
            value.isDarkTheme,
          );
        }).toList(),
        showCheckboxColumn: false,
      );
    });
  }
}

==> verttice_ui-0.0.97.tar.gz: lib/src/widgets/data_factory/table_factory/table_content.dart <==
import "package:collection/collection.dart";
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/const/box_sizes.dart';
import "package:verttice_ui/src/const/icons.dart";
import 'package:verttice_ui/src/state/verttice_ui_state.dart';
import "package:verttice_ui/src/widgets/data_factory/form_factory/form_fields.dart";
import "package:data_table_2/data_table_2.dart";
import "package:flutter/material.dart";
import "package:flutter/scheduler.dart";
import "package:flutter/services.dart";
import "package:provider/provider.dart";
import 'package:verttice_ui/src/widgets/themed_widgets/themed_widgets_index.dart';

import '../common/common_index.dart';
import 'filter_view.dart';

class TableContent extends StatefulWidget {
  final bool showCopyRowButton;
  final List<FormItem> columns;
  final List<Map<String, dynamic>> data;
  final int currentPage;
  final int pageLength;
  final void Function(
    Map<String, dynamic>? rowData,
    int rowIndex,
  ) handleSelectedRow;
  final void Function(List<Map<String, dynamic>>)? selectedListCallback;
  final Function(Map<String, List<FilterField>>) onFilterChanged;
  final Map<String, List<FilterField>> filters;
  final String uniqueIdField;
  final void Function({
    required String field,
    required bool asc,
  })? onSortChanged;

  const TableContent({
    Key? key,
    required this.showCopyRowButton,
    required this.columns,
    required this.data,
    required this.currentPage,
    required this.pageLength,
    required this.handleSelectedRow,
    required this.filters,
    required this.onFilterChanged,
    required this.uniqueIdField,
    this.onSortChanged,
    this.selectedListCallback,
  }) : super(key: key);

  @override
  _TableContentState createState() => _TableContentState();
}

class _TableContentState extends State<TableContent> {
  late List<Map<String, dynamic>> _data;
  final List<Map<String, dynamic>> _selctedRows = [];
  late final List<FormItem> _columns = [];
  late double _tableWidth;
  late int _currentPage;
  late int _pageLength;
  String? _selectedRow;
  int _currentSortColumn = 0;
  bool _isAscending = true;
  late Map<String, List<FilterField>> _filters;

  @override
  void initState() {
    super.initState();
    _filters = widget.filters;
    _data = widget.data;
    _currentPage = widget.currentPage;
    _pageLength = widget.pageLength;
    if (widget.showCopyRowButton) {
      _columns.add(FormItem(field: NoneFF()));
    }
    if (widget.selectedListCallback != null) {
      _columns.add(FormItem(field: NoneFF()));
    }
    _columns.addAll(
      widget.columns.where((item) => item.field.showOnTable).toList(),
    );
    _tableWidth =
        _columns.map((e) => e.field.tableColumnWidth + 17).toList().sum;
  }

  @override
  void didUpdateWidget(covariant TableContent oldWidget) {
    super.didUpdateWidget(oldWidget);

    if (!const DeepCollectionEquality().equals(_data, widget.data)) {
      _data = widget.data;
    }

    if (!const MapEquality().equals(_filters, widget.filters)) {
      _filters = widget.filters;
    }

    if (_currentPage != widget.currentPage ||
        _pageLength != widget.pageLength) {
      _currentPage = widget.currentPage;
      _pageLength = widget.pageLength;
    }
  }

  @override
  void setState(VoidCallback fn) {
    if (mounted) {
      SchedulerBinding.instance.addPostFrameCallback((_) {
        super.setState(fn);
      });
    }
  }

  List<DataColumn2> _buildColumns({
    required Map<String, List<FilterField>> filters,
  }) {
    final List<DataColumn2> tableColumns = [];
    for (var column in _columns) {
      if (_columns.indexOf(column) == 0 && widget.showCopyRowButton) {
        tableColumns.add(
          const DataColumn2(
            size: ColumnSize.S,
            label: SelectableText(
              "Copy",
            ),
          ),
        );
      } else if (widget.selectedListCallback != null &&
          ((!widget.showCopyRowButton && _columns.indexOf(column) == 0) ||
              _columns.indexOf(column) == 1)) {
        tableColumns.add(
          const DataColumn2(
            size: ColumnSize.S,
            label: SelectableText(
              "Select",
            ),
          ),
        );
      } else if (column.field.showOnTable &&
          column.field.displayName != "" &&
          column.field.name != "") {
        tableColumns.add(
          _builDataColumn(
            column: column,
            filters: filters,
          ),
        );
      }
    }
    return tableColumns;
  }

  DataColumn2 _builDataColumn({
    required FormItem column,
    required Map<String, List<FilterField>> filters,
  }) {
    return DataColumn2(
      size: column.field.tableColumnWidth <= 100
          ? ColumnSize.S
          : column.field.tableColumnWidth <= 150
              ? ColumnSize.M
              : ColumnSize.L,
      onSort: column.field.filterType != FilterTypesFF.number ||
              column.field.filterType != FilterTypesFF.string ||
              column.field.filterType != FilterTypesFF.stringList
          ? null
          : (int columnIndex, _) {
              setState(() {
                _currentSortColumn = columnIndex;
                _isAscending = !_isAscending;

                if (widget.onSortChanged != null) {
                  widget.onSortChanged!(
                    field: _columns[columnIndex].field.name,
                    asc: _isAscending,
                  );
                } else {
                  _handleSortData(
                    _columns[columnIndex],
                    _columns[columnIndex].field.name,
                    _isAscending,
                  );
                }
              });
            },
      label: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          sizedBoxH4,
          SizedBox(
            height: 30,
            child: SelectableText(
              column.field.displayName,
              style: const TextStyle(
                overflow: TextOverflow.fade,
              ),
            ),
          ),
          SizedBox(
            height: 30,
            child: TextButton.icon(
              icon: Tooltip(
                message: "Apply filter",
                child: ThemedIcons.filter(
                  iconStyle: IconStyle(
                    size: 17,
                  ),
                ),
              ),
              label: Builder(
                builder: (context) {
                  final labletext = filters[column.field.name]
                      ?.map(
                        (e) => visualDataProcessor(
                          column,
                          {
                            column.field.name: e.value,
                          },
                          lookForPathToFieldOrItemBuilder: false,
                          igonorItemType: true,
                          context: context,
                        ).value,
                      )
                      .toList()
                      .join(", ");
                  return Tooltip(
                    message: labletext ?? '',
                    child: Text(
                      labletext ?? '',
                    ),
                  );
                },
              ),
              onPressed: () => _filterView(
                column: column,
                filters: filters,
              ),
            ),
          ),
          sizedBoxH2,
        ],
      ),
    );
  }

  Future<void> _filterView({
    required FormItem column,
    required Map<String, List<FilterField>> filters,
  }) async {
    await showDialog(
      context: context,
      useSafeArea: true,
      builder: (BuildContext context) {
        return SimpleDialog(
          insetPadding: const EdgeInsets.all(6),
          contentPadding: const EdgeInsets.all(6),
          children: [
            SizedBox(
              height: 450,
              width: 400,
              child: Scaffold(
                body: Center(
                  child: FilterView(
                    column: column,
                    filter: filters[column.field.name],
                    clearFilterCallback: () {
                      final updatedFilters = filters;
                      updatedFilters.remove(column.field.name);
                      widget.onFilterChanged(updatedFilters);
                    },
                    filterCallback: (List<FilterField> filter) {
                      final updatedFilters = filters;
                      updatedFilters[column.field.name] = filter;
                      widget.onFilterChanged(updatedFilters);
                    },
                  ),
                ),
              ),
            )
          ],
        );
      },
    );
  }

  void _handleSortData(
    FormItem column,
    String name,
    bool desc,
  ) {
    _data.sort((a, b) {
      dynamic aS = a[name] ?? "";
      dynamic bS = b[name] ?? "";
      if (column.field.pathToField != null &&
          column.field.pathToField!.isNotEmpty) {
        aS = fieldFromPathToField(
          column.field.pathToField!,
          a.cast<String, dynamic>(),
        );
        bS = fieldFromPathToField(
          column.field.pathToField!,
          b.cast<String, dynamic>(),
        );
      }

      if (aS == null || aS == "") {
        if (bS == null || bS == "") {
          return 0;
        } else {
          return 1;
        }
      } else {
        if (bS == null || bS == "") {
          return -1;
        }
      }

      var r = desc
          ? bS.toString().compareTo(aS.toString())
          : aS.toString().compareTo(bS.toString());
      if (r != 0) return r;
      return desc
          ? bS.toString().compareTo(aS.toString())
          : aS.toString().compareTo(bS.toString());
    });
  }

  DataRow _dataRowBuilder(
    String? selectedRow,
    int rowIndex,
    Map<String, dynamic> rowData,
    bool isDarkTheme,
  ) {
    final List<DataCell> currenRow = [];

    for (var column in _columns) {
      if (_columns.indexOf(column) == 0 && widget.showCopyRowButton) {
        currenRow.add(
          DataCell(
            _copyRow(rowData),
          ),
        );
      } else if (widget.selectedListCallback != null &&
          ((!widget.showCopyRowButton && _columns.indexOf(column) == 0) ||
              _columns.indexOf(column) == 1)) {
        final val = _selctedRows.firstWhereOrNull(
                (element) => const MapEquality().equals(element, rowData)) !=
            null;
        currenRow.add(
          DataCell(
            Tooltip(
              message: val ? "Uncheck" : "Check",
              child: Checkbox(
                onChanged: (selcted) {
                  setState(() {
                    if (selcted ?? false) {
                      _selctedRows.add(rowData);
                    } else {
                      _selctedRows.remove(rowData);
                    }

                    widget.selectedListCallback!(_selctedRows);
                  });
                },
                value: val,
              ),
            ),
          ),
        );
      } else if (column.field.showOnTable &&
          column.field.displayName != "" &&
          column.field.name != "") {
        final visualData = visualDataProcessor(
          column,
          rowData,
          context: context,
        );

        final widget = visualData.widget;
        currenRow.add(
          DataCell(widget),
        );
      }
    }

    final rowID = rowData[widget.uniqueIdField];

    return DataRow(
      onSelectChanged: (bool? selected) {
        widget.handleSelectedRow(rowData, rowIndex);

        setState(() {
          _selectedRow = rowID;
        });
      },
      color: selectedRow == rowID && rowID != null
          ? MaterialStateProperty.all(
              isDarkTheme ? Colors.black : Colors.grey[400],
            )
          : ((rowIndex % 2) == 0)
              ? MaterialStateProperty.all(
                  isDarkTheme ? Colors.grey[600] : Colors.grey[50],
                )
              : MaterialStateProperty.all(
                  isDarkTheme ? Colors.grey[800] : Colors.grey[200],
                ),
      cells: currenRow,
    );
  }

  Widget _copyRow(Map<String, dynamic> rowData) {
    return Tooltip(
      message: "Copy row",
      child: IconButton(
        onPressed: () async {
          await Clipboard.setData(
            ClipboardData(
              text: widget.columns
                  .map(
                    (e) => "${e.field.displayName}: ${visualDataProcessor(
                      e,
                      rowData,
                      context: context,
                    ).value}",
                  )
                  .toList()
                  .join(',\n'),
            ),
          );
          ThemedToast.info(
            context: context,
            message: "Row copied to clipboard",
          );
        },
        icon: ThemedIcons.copy(),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final dataLength = _data.length;

    final sublist = _data.sublist(
        (_currentPage * _pageLength) > dataLength
            ? dataLength
            : (_currentPage * _pageLength),
        ((_currentPage + 1) * _pageLength) > dataLength
            ? dataLength
            : ((_currentPage + 1) * _pageLength));

    return Consumer<VertticeUIState>(builder: (context, value, _child) {
      return DataTable2(
        showBottomBorder: true,
        scrollController: ScrollController(),
        minWidth: _tableWidth,
        sortColumnIndex: _currentSortColumn,
        sortAscending: _isAscending,
        headingRowHeight: 68,
        dataRowHeight: 55,
        columnSpacing: 1,
        columns: _buildColumns(
          filters: _filters,
        ),
        rows: sublist.mapIndexed((rowIndex, rowData) {
          return _dataRowBuilder(
            _selectedRow,
            rowIndex,
            rowData,
            value.isDarkTheme,
          );
        }).toList(),
        showCheckboxColumn: false,
      );
    });
  }
}
